@@ -79,10 +79,6 @@ For example, the debug source for a function compiled from a file will
 include the pathname of the file and the position of the definition."
   'sb-c::debug-source)
 
-(deftype debug-function ()
-  "Debug function represent static compile-time information about a function."
-  'sb-c::compiled-debug-fun)
-
 (declaim (ftype (sfunction (function) debug-info) function-debug-info))
 (defun function-debug-info (function)
   (let* ((function-object (%fun-fun function))
@@ -97,12 +93,6 @@ include the pathname of the file and the position of the definition."
 (defun debug-info-source (debug-info)
   (sb-c::debug-info-source debug-info))
 
-(declaim (ftype (sfunction (t debug-info) debug-function) debug-info-debug-function))
-(defun debug-info-debug-function (function debug-info)
-  (sb-di::compiled-debug-fun-from-pc debug-info
-                                     (sb-di::function-start-pc-offset function)
-                                     t))
-
 (defun valid-function-name-p (name)
   "True if NAME denotes a valid function name, ie. one that can be passed to
 FBOUNDP."
@@ -459,10 +449,10 @@ If an unsupported TYPE is requested, the function will return NIL.
             (type-of object)))))
 
 (defun find-function-definition-source (function)
-  (let* ((debug-info (function-debug-info function))
-         (debug-source (debug-info-source debug-info))
-         (debug-fun (debug-info-debug-function function debug-info))
-         (tlf (if debug-fun (sb-c::compiled-debug-fun-tlf-number debug-fun))))
+  (let* ((debug-source (debug-info-source (function-debug-info function)))
+         (debug-fun (sb-di::fun-debug-fun function))
+         (tlf (sb-c::compiled-debug-fun-tlf-number
+               (sb-di::compiled-debug-fun-compiler-debug-fun debug-fun))))
     (make-definition-source
      :pathname
      (when (stringp (debug-source-namestring debug-source))
@@ -471,7 +461,12 @@ If an unsupported TYPE is requested, the function will return NIL.
      (if tlf
          (elt (sb-c::debug-source-start-positions debug-source) tlf))
      :form-path (if tlf (list tlf))
-     :form-number (sb-c::compiled-debug-fun-form-number debug-fun)
+     :form-number (handler-case (sb-di::code-location-form-number
+                                 (sb-di::debug-fun-start-location debug-fun))
+                    (sb-di::unknown-code-location (cond)
+                      (declare (ignore cond))
+                      (sb-c::compiled-debug-fun-blocks
+                       (sb-di::compiled-debug-fun-compiler-debug-fun debug-fun))))
      :file-write-date (debug-source-created debug-source)
      :plist (sb-c::debug-source-plist debug-source))))
 
@@ -580,14 +575,22 @@ or a method combination name."
 
 (defun find-function-callees (function)
   "Return functions called by FUNCTION."
-  (declare (simple-fun function))
-  (let ((callees '()))
-    (map-code-constants
-     (fun-code-header function)
-     (lambda (obj)
-       (when (fdefn-p obj)
-         (push (fdefn-fun obj) callees))))
-    callees))
+  (if (typep function 'generic-function)
+      (loop for method in (sb-mop:generic-function-methods function)
+            for method-fun = (sb-mop:method-function method)
+            append (find-function-callees
+                    (if (typep (%fun-name method-fun) '(cons (eql sb-pcl::call)))
+                        (sb-kernel:%closure-index-ref  method-fun 0)
+                        method-fun)))
+      (let ((callees '()))
+        (map-code-constants
+         (fun-code-header (sb-kernel:%fun-fun function))
+         (lambda (obj)
+           (when (fdefn-p obj)
+             (let ((fun (fdefn-fun obj)))
+               (when fun
+                 (push fun callees))))))
+        callees)))
 
 (defun find-function-callers (function &optional (spaces '(:read-only :static
                                                            :dynamic
@@ -604,6 +607,7 @@ or a method combination name."
 
 ;;; XREF facility
 
+#-(and system-tlabs (not mark-region-gc))
 (defun collect-xref (wanted-kind wanted-name)
   (let ((result '()))
     (sb-c:map-simple-funs
@@ -636,6 +640,84 @@ or a method combination name."
           xrefs))))
     result))
 
+#+(and system-tlabs (not mark-region-gc))
+(progn
+(sb-ext:defglobal *codeblob-cache* nil)
+(flet ((gather-code (stamp) ; = the value of sb-vm::*code-alloc-count*
+         ;; Remove unreachable functions.
+         (sb-ext:gc :full t)
+         (let ((arena (sb-vm:new-arena (* 2 1024 1024)))
+               (result))
+           ;; Can allocate inside an arena while holding without-gcing in sb-vm:map-code-objects
+           ;; Anyway this approach is silly because Lisp should maintain at all times
+           ;; a binary-searchable tree of all code which would solve all problems
+           ;; related to finding a codeblob from a PC without relying on whatever
+           ;; a particular GC implementation exposes in terms of linearly searchable
+           ;; ranges of memory. immobile-space does maintain such a tree. Of course the tree
+           ;; should also _weakly_ reference all code, and should be usable for xref
+           ;; and other consumers beside the debugger. And it should come with a pony too.
+           (unwind-protect
+                (sb-vm:with-arena (arena)
+                  ;; No filtering since we want this to pertain to all COLLECT-XREFS calls
+                  (sb-vm:map-code-objects (lambda (code) (push code result))))
+             ;; arenas are not suitable for returning memoized data
+             (setq result (coerce result 'vector))
+             (sb-vm:destroy-arena arena))
+           (setf *codeblob-cache* (cons stamp (sb-ext:make-weak-pointer result)))
+           result)))
+(defun collect-xref (wanted-kind wanted-name)
+  (let* ((current-stamp sb-vm::*code-alloc-count*)
+         (all-code
+          ;; this is not an attempt to be 100% correct in observing an up-to-date
+          ;; snapshot at a point in time.  It's close enough though.
+          ;; I can't imagine that users are clamoring for a perfect solution to
+          ;; racing threads and XREFing jit-compiled code.
+          (or (let ((cache *codeblob-cache*))
+                (and (eql (car cache) current-stamp)
+                     (sb-ext:weak-pointer-value (cdr cache))))
+              (loop ; expect exactly 1 iteration
+               (let ((vector (gather-code current-stamp))
+                     (new-stamp sb-vm::*code-alloc-count*))
+                 (if (eq new-stamp current-stamp) ; say it's done
+                     (return vector)
+                     (setq current-stamp new-stamp))))))
+         (funs))
+    (dovector (code all-code)
+      (dotimes (i (code-n-entries code))
+        (let ((fun (%code-entry-point code i)))
+          (binding* ((xrefs (%simple-fun-xrefs fun) :exit-if-null))
+            (sb-c:map-packed-xref-data
+             (lambda (xref-kind xref-name xref-form-number)
+               (when (and (eq xref-kind wanted-kind)
+                          (equal xref-name wanted-name))
+                 (push (cons fun xref-form-number) funs)))
+             xrefs)))))
+    (let (result)
+      (loop for (fun . xref-form-number) in funs
+            do
+            (let ((source-location (find-function-definition-source fun)))
+              ;; Use the more accurate source path from the xref
+              ;; entry.
+              (setf (definition-source-form-number source-location) xref-form-number)
+              (let* ((name (sb-c::%fun-name fun))
+                     (name (cond ((typep name '(cons (eql sb-c:deftransform)))
+                                  (let* ((fun-name (second name))
+                                         (info (sb-int:info :function :info fun-name))
+                                         (transform (and info
+                                                         (find fun (sb-c::fun-info-transforms info)
+                                                               :key #'sb-c::transform-function))))
+                                    (if transform
+                                        (append name
+                                                (let* ((type (sb-c::transform-type transform))
+                                                       (type-spec (type-specifier type)))
+                                                  (and (sb-kernel:fun-type-p type)
+                                                       (list (second type-spec)))))
+                                        name)))
+                                 (t
+                                  name))))
+                (pushnew (cons name source-location) result :test #'equalp))))
+      result)))))
+
 (defun who-calls (function-name)
   "Use the xref facility to search for source locations where the
 global function named FUNCTION-NAME is called. Returns a list of
