@@ -22,7 +22,7 @@
   (addr system-area-pointer)
   (bytes unsigned))
 
-(define-load-time-global *allocator-mutex* (sb-thread:make-mutex :name "Allocator"))
+(define-load-time-global *allocator-mutex* nil)
 
 (defun allocate-static-vector (widetag length words)
   (declare (type (unsigned-byte #.n-widetag-bits) widetag)
@@ -114,7 +114,7 @@
                         aligned-nwords
                         header))))
 
-(defun make-immobile-symbol (name)
+(defun %alloc-immobile-symbol (name)
   (let ((symbol (truly-the symbol
                  (or #+x86-64 (%primitive !fast-alloc-immobile-symbol)
                      (alloc-immobile-fixedobj
@@ -176,15 +176,22 @@
               (let ((oldval (cas *dynspace-codeblob-tree* tree newtree)))
                 (if (eq oldval tree) (return) (setq tree oldval))))))))
 
+(defglobal *code-alloc-count* 0) ; frlock: bump once on entry, again on exit
+(declaim (fixnum *code-alloc-count*))
+
 ;;; Allocate a code component with BOXED words in the header
 ;;; followed by UNBOXED bytes of raw data.
 ;;; BOXED must be the exact count of boxed words desired. No adjustments
 ;;; are made for alignment considerations or the fixed slots.
+;;; FIXME: it's not necessary that there be two different locks: *allocator-mutex*
+;;; and code_allocator_lock. The Lisp mutex should subsume the C mutex,
+;;; and we should acquire the Lisp one around the entire body of this function.
 (defun allocate-code-object (space boxed unboxed)
   (declare (ignorable space))
   (let* ((total-words
            (the (unsigned-byte 22) ; Enforce limit on total words as well
                 (align-up (+ boxed (ceiling unboxed n-word-bytes)) 2))))
+    (atomic-incf *code-alloc-count*)
     #+immobile-code
     (when (member space '(:immobile :auto))
       (let (addr code holder)
@@ -211,6 +218,7 @@
           (let ((tree *immobile-codeblob-tree*))
             (loop (when (eq tree (setq tree (cas *immobile-codeblob-tree* tree
                                                  (sb-brothertree:insert addr tree))))
+                    (atomic-incf *code-alloc-count*)
                     (return-from allocate-code-object (values code total-words)))))))
       (when (eq space :immobile)
         (error "Immobile code space exhausted")))
@@ -232,6 +240,7 @@
     ;; of the object so that we can find the function table.
     ;; But what about other things that create code objects?
     ;; It could be a subtle source of nondeterministic core images.
+      (atomic-incf *code-alloc-count*)
       (values code total-words))))
 
 ;; The freelist can only be read while holding a mutex because the codeblobs
@@ -277,3 +286,16 @@
           (alien-funcall tlsf-unalloc-codeblob tlsf-control addr)
           (setf (car scratchpad) (pop-1)))))
     t))
+
+#+permgen
+(defun sb-kernel::allocate-permgen-layout (nwords)
+  (with-system-mutex (*allocator-mutex* :without-gcing t)
+    (let ((freeptr *permgen-space-free-pointer*))
+      (setf *permgen-space-free-pointer*
+            ;; round-to-odd, add the header word
+            (sap+ freeptr (ash (1+ (logior nwords 1)) word-shift)))
+      (aver (<= (sap-int *permgen-space-free-pointer*)
+                (+ permgen-space-start permgen-space-size)))
+      (setf (sap-ref-word freeptr 0)
+            (logior (ash nwords instance-length-shift) instance-widetag))
+      (%make-lisp-obj (sap-int (sap+ freeptr instance-pointer-lowtag))))))
