@@ -13,6 +13,15 @@
 
 (in-package "SB-IMPL")
 
+;;; Like DEFUN, but hides itself in the backtrace. This is meant for
+;;; trivial functions which just do some argument parsing and call
+;;; ERROR for real. Hence, having them and their locals in the
+;;; backtrace would add no useful information pertaining to the error.
+(defmacro define-error-wrapper (name lambda-list &body body)
+  `(defun ,name (,@lambda-list
+                 &aux #-sb-xc-host (sb-debug:*stack-top-hint* (or sb-debug:*stack-top-hint* ',name)))
+     ,@body))
+
 ;;; FIXME: a lot of places in the code that should use ARRAY-RANGE
 ;;; instead use INDEX and vice-versa, but the thing is, we have
 ;;; a ton of fenceposts errors all over the place regardless.
@@ -104,7 +113,7 @@
 ;;; have cycles and isn't too large.
 (defun coalesce-tree-p (x)
   (let ((depth-limit 12)
-        (size-limit (expt 2 25)))
+        (size-limit (expt 2 12)))
     (declare (fixnum size-limit))
     (and (consp x)
          (labels ((safe-cddr (cons)
@@ -171,7 +180,9 @@
 ;;;; comment from CMU CL: "the ultimate collection macro..."
 
 ;;; helper function for COLLECT, which becomes the expander of the
-;;; MACROLET definitions created by COLLECT if collecting a list.
+;;; MACROLET definitions created by COLLECT if collecting a list
+;;; and an INITIAL-VALUE was specified, so we don't know at each step
+;;; whether the tail is NIL or not.
 ;;; N-TAIL is the pointer to the current tail of the list,  or NIL
 ;;; if the list is empty.
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
@@ -198,37 +209,67 @@
 ;;; the collection macro with no arguments.
 ;;;
 ;;; INITIAL-VALUE is the value that the collection starts out with,
-;;; which defaults to NIL. FUNCTION is the function which does the
+;;; which defaults to NIL. COLLECTOR is the function which does the
 ;;; collection. It is a function which will accept two arguments: the
 ;;; value to be collected and the current collection. The result of
 ;;; the function is made the new value for the collection. As a
-;;; totally magical special-case, FUNCTION may be COLLECT, which tells
-;;; us to build a list in forward order; this is the default. If an
-;;; INITIAL-VALUE is supplied for COLLECT, the stuff will be RPLACD'd
-;;; onto the end. Note that FUNCTION may be anything that can appear
-;;; in the functional position, including macros and lambdas.
+;;; special-case, omitting COLLECTOR causes a list to be built in forward
+;;; order. If INITIAL-VALUE is supplied for the default usage, new items
+;;; will be RPLACD'd onto the end.
+;;; Note that COLLECTOR may be anything that can appear in the functional
+;;; position, including macros and lambdas.
+;;; Also note that invocation of the collector macro for effect, i.e. other
+;;; than with 0 arguments, is not prescribed to have any particular value.
 (defmacro collect (collections &body body)
   (let ((macros ())
         (binds ())
+        (dx ())
         (ignores ()))
     (dolist (spec collections)
-      (destructuring-bind (name &optional default collector
-                                &aux (n-value (copy-symbol name))) spec
-        (push `(,n-value ,default) binds)
+      (destructuring-bind (name &optional initial-value (collector nil collectorp)
+                                &aux (n-value (copy-symbol name)))
+          spec
+        (push `(,n-value ,(if (or initial-value collectorp)
+                              initial-value
+                              `(#-sb-xc-host unaligned-dx-cons
+                                #+sb-xc-host list
+                                nil)))
+              binds)
         (let ((macro-body
-               (if (or (null collector) (eq collector 'collect))
-                   (let ((n-tail (gensymify* name "-TAIL")))
-                     (push n-tail ignores)
-                     (push `(,n-tail ,(if default `(last ,n-value))) binds)
-                     `(collect-list-expander ',n-value ',n-tail args))
+               (cond
+                 (collectorp
                    ``(progn
                        ,@(mapcar (lambda (x)
                                    `(setq ,',n-value (,',collector ,x ,',n-value)))
                                  args)
-                       ,',n-value))))
+                       ,',n-value))
+                 ((not initial-value)
+                  ;; Use a dummy cons to skip the test for TAIL being NIL with each
+                  ;; inserted item.
+                  (push n-value dx)
+                  (let ((n-tail (gensymify* name "-TAIL")))
+                    (push n-tail ignores)
+                    (push `(,n-tail ,n-value) binds)
+                    `(if args
+                         `(progn
+                            ,@(mapcar (lambda (x)
+                                        `(setf ,',n-tail (setf (cdr ,',n-tail)
+                                                               (list ,x))))
+                                      args))
+                         `(cdr ,',n-value))))
+                 ;; collecting a list given a list to start with.
+                 ;; It's possible to use the "fancy" strategy to avoid testing for NIL
+                 ;; at each step but I choose not to.  The initializer would have to be
+                 ;; (cons nil initial-value). It's unimportant.
+                 (initial-value
+                  (let ((n-tail (gensymify* name "-TAIL")))
+                    (push n-tail ignores)
+                    (push `(,n-tail (last ,n-value)) binds)
+                    `(collect-list-expander ',n-value ',n-tail args))))))
           (push `(,name (&rest args) ,macro-body) macros))))
     `(macrolet ,macros
        (let* ,(nreverse binds)
+         ,@(if dx `((declare (dynamic-extent ,@dx))))
          ;; Even if the user reads each collection result,
          ;; reader conditionals might statically eliminate all writes.
          ;; Since we don't know, all the -n-tail variable are ignorable.
@@ -237,16 +278,6 @@
 
 ;;; Functions for compatibility sake:
 
-(defun memq (item list)
-  "Return tail of LIST beginning with first element EQ to ITEM."
-  (declare (inline member))
-  (member item list :test #'eq))
-
-(defun assq (item alist)
-  "Return the first pair of alist where item is EQ to the key of pair."
-  (declare (inline assoc))
-  (assoc item alist :test #'eq))
-
 ;;; Delete just one item
 (defun delq1 (item list)
   (do ((prev nil x)
@@ -467,6 +498,8 @@ NOTE: This interface is experimental and subject to change."
   (dolist (name *cache-vector-symbols*)
     (set name nil)))
 
+(defmacro sb-int-package () (find-package "SB-INT"))
+
 ;; Make a new hash-cache and optionally create the statistics vector.
 (defun alloc-hash-cache (size symbol)
   (declare (type index size))
@@ -485,8 +518,7 @@ NOTE: This interface is experimental and subject to change."
                  ;; it is inconsequential to performance.
                  (if *profile-hash-cache*
                      `(let ((statistics
-                             (let ((*package* (sb-xc:symbol-package symbol)))
-                               (symbolicate symbol "STATISTICS"))))
+                             (package-symbolicate (sb-int-package) symbol "-STATS")))
                         (unless (boundp statistics)
                           (set! statistics
                                 (make-array 3 :element-type 'fixnum
@@ -533,9 +565,9 @@ NOTE: This interface is experimental and subject to change."
   (def 5)
   (def 6))
 
-(defmacro !define-hash-cache (name args aux-vars
+(defmacro !define-hash-cache (name args
                               &key hash-function hash-bits memoizer
-                              flush-function (values 1))
+                              (values 1))
   (declare (ignore memoizer))
   (dolist (arg args)
     ;; Each arg is (VARNAME EQUIVALENCE-TEST &OPTIONAL EXPR-TO-CACHE)
@@ -543,43 +575,71 @@ NOTE: This interface is experimental and subject to change."
     ;; (the default is to store each argument as-is)
     (unless (<= 2 (length arg) 3)
       (error "bad argument spec: ~S" arg)))
-  (assert (typep hash-bits '(integer 5 14))) ; reasonable bounds
-  (let* ((fun-name (symbolicate "!" name "-MEMO-WRAPPER"))
-         (var-name (symbolicate "**" name "-CACHE-VECTOR**"))
+  ;; Disallow >12 hash bits. Supposing a 32-bit machine, a ctype hash uses 5
+  ;; reserved bits for the type-class index, and then we need 2 independent
+  ;; choices of hash, so supposing that's 12 bits each, we're at 29 bits.
+  ;; But I want to take 2 bits for flags, so that's 31 bits.
+  (assert (typep hash-bits '(integer 5 12))) ; reasonable bounds
+  (unless (integerp values) ; non-numeric was only for CTYPE-OF-ARRAY
+    (error "Values ~S unsupported" values))
+  (let* ((var-name (symbolicate "**" name "-CACHE-VECTOR**"))
+         (statistics-name (package-symbolicate (sb-int-package) var-name "-STATS"))
+         (cache-type `(simple-vector ,(ash 1 hash-bits))))
+    `(progn
+       (eval-when (:compile-toplevel :execute)
+         (setf (get ',name '!cache-definition)
+               (list ',hash-bits ',hash-function ',args ',values)))
+       (pushnew ',var-name *cache-vector-symbols*)
+       (define-load-time-global ,var-name nil)
+       ,@(when *profile-hash-cache*
+           `((declaim (type (simple-array fixnum (3)) ,statistics-name))
+             (defvar ,statistics-name)))
+       (declaim (type (or null ,cache-type) ,var-name))
+       (defun ,(symbolicate name "-CACHE-CLEAR") () (setq ,var-name nil))
+       ',var-name)))
+
+(eval-when (:compile-toplevel :execute) ; leave this out of the core image
+(#+sb-xc-host defmacro #-sb-xc-host sb-xc:defmacro
+ with-cache ((fun-name &rest actual-args) &body computation)
+  (let* ((var-name (package-symbolicate (cl:symbol-package fun-name)
+                                        "**" fun-name "-CACHE-VECTOR**"))
+         (cache-params (get fun-name '!cache-definition))
          (statistics-name
           (when *profile-hash-cache*
-            (symbolicate var-name "STATISTICS")))
-         (nargs (length args))
+            (package-symbolicate (sb-int-package) var-name "-STATS")))
+         (hash-bits (first cache-params))
+         (hashfun (second cache-params))
+         (arg-specs (third cache-params))
+         (nargs (length arg-specs))
+         (values (fourth cache-params))
          (size (ash 1 hash-bits))
          (hashval (make-symbol "HASH"))
          (cache (make-symbol "CACHE"))
          (entry (make-symbol "LINE"))
-         (thunk (make-symbol "THUNK"))
-         (arg-vars (mapcar #'first args))
          (nvalues (if (listp values) (length values) values))
          (result-temps
           (if (listp values)
               values ; use the names provided by the user
               (loop for i from 1 to nvalues ; else invent some names
                     collect (make-symbol (format nil "R~D" i)))))
-         (temps (append (mapcar (lambda (x) (make-symbol (string x)))
-                                arg-vars)
-                        result-temps))
+         (temps (append (mapcar #'copy-symbol actual-args) result-temps))
          ;; Mnemonic: (FIND x SEQ :test #'f) calls f with x as the LHS
-         (tests (mapcar (lambda (spec temp) ; -> (EQx ARG #:ARG)
-                          `(,(cadr spec) ,(car spec) ,temp))
-                        args temps))
+         (tests (mapcar (lambda (spec temp actual) ; -> (EQx #:ARG ARG)
+                          `(,(cadr spec) ,temp ,actual))
+                        arg-specs temps actual-args))
          (cache-type `(simple-vector ,size))
          (line-type (let ((n (+ nargs nvalues)))
                       (if (<= n 3) 'cons `(simple-vector ,n))))
-         (bind-hashval
-          `((,hashval (the sb-xc:fixnum
-                           (funcall ,hash-function ,@arg-vars)))
-            (,cache ,var-name)))
+         ;; It's not really necessary to factor out PROBE-IT,
+         ;; but it used to be possible to probe for an entry to forcefully
+         ;; evict (unmemoize) it. That capability is no longer needed.
          (probe-it
-          (lambda (ignore action)
+          (lambda (action)
             `(when ,cache
-               (let ((,hashval ,hashval) ; gets clobbered in probe loop
+               ;; Shift out 2 low bits because they won't be as random as other bits,
+               ;; at least not for single-argument cached functions. They might be OK
+               ;; for functions that involve mixing.
+               (let ((,hashval (ash ,hashval -2)) ; gets clobbered in probe loop
                      (,cache (truly-the ,cache-type ,cache)))
                  ;; FIXME: redundant?
                  (declare (type sb-xc:fixnum ,hashval))
@@ -588,8 +648,8 @@ NOTE: This interface is experimental and subject to change."
                               (svref ,cache
                                      (ldb (byte ,hash-bits 0) ,hashval))))
                          (unless (eql ,entry 0)
-                           (locally (declare (type ,line-type ,entry))
-                             (let* ,(case (length temps)
+                           (let* ((,entry (truly-the ,line-type ,entry))
+                                  ,@(case (length temps)
                                      (2 `((,(first temps) (car ,entry))
                                           (,(second temps) (cdr ,entry))))
                                      (3 (let ((arg-temp (gensym "ARGS")))
@@ -599,63 +659,50 @@ NOTE: This interface is experimental and subject to change."
                                              (car (truly-the cons ,arg-temp)))
                                             (,(third temps) (cdr ,arg-temp)))))
                                      (t (loop for i from 0 for x in temps
-                                              collect `(,x (svref ,entry ,i)))))
-                               ,@ignore
-                               (when (and ,@tests) ,action))))
-                         (setq ,hashval (ash ,hashval ,(- hash-bits)))))))))
-         (fun
-          `(defun ,fun-name (,thunk ,@arg-vars ,@aux-vars)
-             ,@(when *profile-hash-cache* ; count seeks
-                 `((when (boundp ',statistics-name)
-                     (incf (aref ,statistics-name 0)))))
-             (let ,bind-hashval
-               ,(funcall probe-it nil
-                         `(return-from ,fun-name (values ,@result-temps)))
-               (multiple-value-bind ,result-temps (funcall ,thunk)
-                 (let ((,entry
-                        (,(hash-cache-line-allocator (+ nargs nvalues))
-                         ,@(mapcar (lambda (spec) (or (caddr spec) (car spec)))
-                                   args)
-                         ,@result-temps))
-                       (,cache
-                        (truly-the ,cache-type
-                         (or ,cache (alloc-hash-cache ,size ',var-name))))
-                       (idx1 (ldb (byte ,hash-bits 0) ,hashval))
-                       (idx2 (ldb (byte ,hash-bits ,hash-bits) ,hashval)))
-                   ,@(when *profile-hash-cache*
-                       `((incf (aref ,statistics-name 1)))) ; count misses
+                                              collect `(,x (svref ,entry ,i))))))
+                               (when (and ,@tests) ,action)))
+                         (setq ,hashval (ash ,hashval ,(- hash-bits))))))))))
+    `(let ((,hashval (the sb-xc:fixnum (funcall ,hashfun ,@actual-args)))
+           (,cache ,var-name))
+       #-sb-xc-host (declare (optimize (sb-c::insert-array-bounds-checks 0)))
+       ,@(when *profile-hash-cache* ; count seeks
+           `((when (boundp ',statistics-name) (incf (aref ,statistics-name 0)))))
+       (block seek
+         ,(funcall probe-it `(return-from seek (values ,@result-temps)))
+         (multiple-value-bind ,result-temps (progn ,@computation)
+           ;; Decide if cacheable result. Lambda expressions and strings always are.
+           ;; Parsing a type specifier nonlocally exits from COMPUTATION if it
+           ;; wants not to cache. And of course CTYPE-OF always returns a good type.
+           ;; It's unfortunate that this macro macro bakes in such knowledge.
+           (when ,(if (member fun-name '(sb-format::tokenize-control-string
+                                         sb-alien::coerce-to-interpreted-function
+                                         values-specifier-type
+                                         ctype-of))
+                      t
+                      `(and ,@(mapcar (lambda (x) `(sb-kernel::ok-to-memoize-p ,x))
+                                      actual-args)))
+             (let ((,entry (,(hash-cache-line-allocator (+ nargs nvalues))
+                            ,@(mapcar (lambda (spec actual) (or (caddr spec) actual))
+                                      arg-specs actual-args)
+                            ,@result-temps))
+                   (,cache (truly-the ,cache-type
+                                      (or ,cache (alloc-hash-cache ,size ',var-name))))
+                   (idx1 (ldb (byte ,hash-bits 2) ,hashval))
+                   (idx2 (ldb (byte ,hash-bits ,(+ 2 hash-bits)) ,hashval)))
+               ,@(when *profile-hash-cache*
+                   `((incf (aref ,statistics-name 1)))) ; count misses
                    ;; Why a barrier: the pointer to 'entry' (a cons or vector)
                    ;; MUST NOT be observed by another thread before its cells
                    ;; are filled. Equally bad, the 'output' cells in the line
                    ;; could be 0 while the 'input' cells matched something.
-                   (sb-thread:barrier (:write))
-                   (cond ((eql (svref ,cache idx1) 0)
-                          (setf (svref ,cache idx1) ,entry))
-                         ((eql (svref ,cache idx2) 0)
-                          (setf (svref ,cache idx2) ,entry))
-                         (t
-                           ,@(when *profile-hash-cache* ; count evictions
-                               `((incf (aref ,statistics-name 2))))
-                           (setf (svref ,cache idx1) ,entry))))
-                 (values ,@result-temps))))))
-    `(progn
-       (pushnew ',var-name *cache-vector-symbols*)
-       (define-load-time-global ,var-name nil)
-       ,@(when *profile-hash-cache*
-           `((declaim (type (simple-array fixnum (3)) ,statistics-name))
-             (defvar ,statistics-name)))
-       (declaim (type (or null ,cache-type) ,var-name))
-       (defun ,(symbolicate name "-CACHE-CLEAR") () (setq ,var-name nil))
-       ,@(when flush-function
-           `((defun ,flush-function ,arg-vars
-               (let ,bind-hashval
-                 ,(funcall probe-it
-                   `((declare (ignore ,@result-temps)))
-                   `(return (setf (svref ,cache
-                                         (ldb (byte ,hash-bits 0) ,hashval))
-                                  0)))))))
-       (declaim (inline ,fun-name))
-       ,fun)))
+               (sb-thread:barrier (:write))
+               (setf (svref ,cache (cond ((eql (svref ,cache idx1) 0) idx1)
+                                         ((eql (svref ,cache idx2) 0) idx2)
+                                         (t ,@(when *profile-hash-cache* ; count evictions
+                                                `((incf (aref ,statistics-name 2))))
+                                            idx1)))
+                     ,entry)))
+           (values ,@result-temps)))))))
 
 ;;; some syntactic sugar for defining a function whose values are
 ;;; cached by !DEFINE-HASH-CACHE
@@ -682,10 +729,7 @@ NOTE: This interface is experimental and subject to change."
 ;;; Since we don't have caches that aren't in direct support of DEFUN-CACHED
 ;;; - did we ever? - this should be possible to change.
 ;;;
-(defmacro defun-cached ((name &rest options &key
-                              (memoizer (make-symbol "MEMOIZE")
-                                        memoizer-supplied-p)
-                              &allow-other-keys)
+(defmacro defun-cached ((name &rest options &key memoizer &allow-other-keys)
                         args &body body-decls-doc)
   (binding* (((forms decls doc) (parse-body body-decls-doc t))
              ((inputs aux-vars)
@@ -694,20 +738,17 @@ NOTE: This interface is experimental and subject to change."
                     (values (ldiff args aux) aux)
                     (values args nil))))
              (arg-names (mapcar #'car inputs)))
+    (assert (not aux-vars)) ; was only for CTYPE-OF-ARRAY
     `(progn
-        (!define-hash-cache ,name ,inputs ,aux-vars ,@options)
+        (!define-hash-cache ,name ,inputs ,@options)
         (defun ,name ,arg-names
           ,@decls
           ,@(if doc (list doc))
-          (macrolet ((,memoizer (&body body)
-                       ;; We don't need (DX-FLET ((,thunk () ,@body)) ...)
-                       ;; This lambda is a single-use local call within
-                       ;; the inline memoizing wrapper.
-                       `(,',(symbolicate "!" name "-MEMO-WRAPPER")
-                         (lambda () ,@body) ,@',arg-names)))
-             ,@(if memoizer-supplied-p
-                   forms
-                   `((,memoizer ,@forms))))))))
+          ,(if memoizer
+               `(macrolet ((,memoizer (&body body)
+                             `(with-cache ,',(list* name arg-names) ,@body)))
+                  ,@forms)
+               `(with-cache (,name ,@arg-names) ,@forms))))))
 
 (defun list-elts-eq (x y)
   (loop (when (eq x y) (return t))
@@ -757,8 +798,7 @@ NOTE: This interface is experimental and subject to change."
 (deftype function-name () '(satisfies legal-fun-name-p))
 
 ;;; Signal an error unless NAME is a legal function name.
-(defun legal-fun-name-or-type-error (name)
-  #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
+(define-error-wrapper legal-fun-name-or-type-error (name)
   (unless (legal-fun-name-p name)
     (error 'simple-type-error
            :datum name
@@ -846,11 +886,22 @@ NOTE: This interface is experimental and subject to change."
 ;;; with this should reduce the size of the system by enough to be
 ;;; worthwhile.)
 (defmacro aver (expr)
-  `(unless ,expr
-     (%failed-aver ',expr)))
+  ;; Don't hold on to symbols, helping shake-packages.
+  (labels ((replace-symbols (expr)
+             (typecase expr
+               (null expr)
+               (symbol
+                (symbol-name expr))
+               (cons
+                (cons (replace-symbols (car expr))
+                      (replace-symbols (cdr expr))))
+               (t
+                expr))))
+    `(unless ,expr
+       (%failed-aver ',(replace-symbols expr)))))
 
 (defun %failed-aver (expr)
-  (bug "~@<failed AVER: ~2I~_~S~:>" expr))
+  (bug "~@<failed AVER: ~2I~_~A~:>" expr))
 
 (defun bug (format-control &rest format-arguments)
   (error 'bug
@@ -1128,8 +1179,7 @@ NOTE: This interface is experimental and subject to change."
 ;;; Without a proclaimed type, the call is "untrusted" and so the compiler
 ;;; would generate a post-call check that the function did not return.
 (declaim (ftype (function (t t t t t) nil) deprecation-error))
-(defun deprecation-error (software version namespace name replacements)
-  #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
+(define-error-wrapper deprecation-error (software version namespace name replacements)
   (error 'deprecation-error
          :namespace namespace
          :name name
@@ -1246,7 +1296,6 @@ NOTE: This interface is experimental and subject to change."
                                 ((or (listp id) ; must be a type-specifier
                                      (memq id '(special ignorable ignore
                                                 dynamic-extent
-                                                truly-dynamic-extent
                                                 sb-c::constant-value
                                                 sb-c::no-constraints))
                                      (info :type :kind id))
@@ -1302,7 +1351,7 @@ NOTE: This interface is experimental and subject to change."
 
 (defun call-with-sane-io-syntax (function)
   (declare (type function function))
-  #-sb-xc-host (declare (dynamic-extent function)) ; "unable"
+  (declare (dynamic-extent function))
   ;; force BOUNDP to be tested by declaring maximal safety
   ;; in case unsafe code really screwed things up.
   (declare (optimize (safety 3)))
@@ -1337,8 +1386,7 @@ NOTE: This interface is experimental and subject to change."
 '(defun show-hash-cache-statistics ()
   (flet ((cache-stats (symbol)
            (let* ((name (string symbol))
-                  (statistics (let ((*package* (sb-xc:symbol-package symbol)))
-                                (symbolicate symbol "STATISTICS")))
+                  (statistics (package-symbolicate (sb-int-package) symbol "-STATS"))
                   (prefix
                    (subseq name 0 (- (length name) (length "VECTOR**")))))
              (values (if (boundp statistics)
@@ -1430,5 +1478,14 @@ NOTE: This interface is experimental and subject to change."
               (eq (symbol-value x) x))))
     (cons nil)
     (t t)))
+
+(declaim (inline first-bit-set))
+(defun first-bit-set (x)
+  #+(and x86-64 (not sb-xc-host))
+  (truly-the (values (mod #.sb-vm:n-word-bits) &optional)
+             (%primitive sb-vm::unsigned-word-find-first-bit (the word x)))
+  #-(and x86-64 (not sb-xc-host))
+  (1- (integer-length (logand x (- x)))))
+
 
 (defvar *top-level-form-p* nil)
