@@ -392,10 +392,18 @@
        (result '() (cons initial-element result)))
       ((<= count 0) result)
     (declare (type index count))))
+
+(defun %sys-make-list (size initial-element)
+  (declare (type index size)
+           (sb-c::tlab :system))
+  (do ((count size (1- count))
+       (result '() (cons initial-element result)))
+      ((<= count 0) result)
+    (declare (type index count))))
 
 (defun append (&rest lists)
   "Construct and return a list by concatenating LISTS."
-  (let* ((result (list nil))
+  (let* ((result (unaligned-dx-cons nil))
          (tail result)
          (index 0)
          (length (length lists))
@@ -1327,6 +1335,7 @@
         (if accumulate
             (cdr ret-list)
             non-acc-result))
+    (declare (dynamic-extent ret-list))
     (do ((l arglists (cdr l))
          (arg args (cdr arg)))
         ((null l))
