@@ -378,9 +378,8 @@
 
 ;;;; the FOP database
 
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  ;; The bottom 5 bits of the opcodes above 128 encode an implicit operand.
-  (defconstant n-ordinary-fops 128))
+;; The bottom 5 bits of the opcodes above 128 encode an implicit operand.
+(defconstant n-ordinary-fops 128)
 
 ;;; a vector indexed by a FaslOP that yields a function which performs
 ;;; the operation. Most functions take 0 arguments - they only manipulate
@@ -441,6 +440,7 @@
                  (error ,(format nil "Not-host fop invoked: ~A" name)))))
        (!%define-fop ',name ,fop-code ,(length operands) ,(if pushp 1 0)))))
 
+(defglobal *fop-name-to-opcode* (make-hash-table))
 (defun !%define-fop (name opcode n-operands pushp)
   (declare (type (mod 4) n-operands))
   (let ((function (svref **fop-funs** opcode)))
@@ -454,7 +454,7 @@
       (when (and existing-opcode (/= existing-opcode opcode))
         (error "multiple codes for fop name ~S: ~D and ~D"
                name opcode existing-opcode)))
-    (setf (get name 'opcode) opcode
+    (setf (gethash name *fop-name-to-opcode*) opcode
           (svref **fop-funs** opcode) (symbol-function name)
           (aref (car **fop-signatures**) opcode) n-operands
           (sbit (cdr **fop-signatures**) opcode) pushp))
@@ -724,8 +724,11 @@
                      n-operands arg1 arg2 arg3)
              (when (functionp function)
                (format *trace-output* " ~35t~(~a~)" (%fun-name function))
-               (when (eql (%fun-name function) 'fop-push)
-                 (format *trace-output* " ~(~A~)" (ref-fop-table fasl-input arg1)))))))))))
+               (case (%fun-name function)
+                 (fop-push (format *trace-output* " ~(~A~)" (ref-fop-table fasl-input arg1)))
+                 (fop-word-integer (format *trace-output* " ~V,'0X" (* 2 sb-vm:n-word-bytes) result))
+                 (fop-byte-integer (format *trace-output* " ~2,'0X" result))
+                 (fop-integer (format *trace-output* " ~X" result)))))))))))
 
 (defun load-as-fasl (stream verbose print)
   (when (zerop (file-length stream))
@@ -784,12 +787,6 @@
         (incf ptr)))
     res))
 
-;;; Symbol-like entities
-(define-fop 49 :not-host (fop-debug-name-marker ((:operands kind)))
-  (ecase kind
-   (1 sb-c::*debug-name-sharp*)
-   (2 sb-c::*debug-name-ellipsis*)))
-
 (define-fop 45 :not-host (fop-layout ((:operands depthoid flags length)
                                        name bitmap inherits))
   (decf depthoid) ; was bumped by 1 since non-stack args can't encode negatives
@@ -849,17 +846,7 @@
            (multiple-value-bind (name length elt-type)
                (read-symbol-name length+flag fasl-input)
              (push-fop-table (%intern name length package elt-type t inherited)
-                             fasl-input)))
-         (ensure-hashed (symbol)
-           ;; ENSURE-SYMBOL-HASH when vop-translated is flushable since it is
-           ;; conceptually just a slot reader, however its actual effect is to fill in
-           ;; the hash if absent, so it's not quite flushable when called expressly
-           ;; to fill in the slot. In this case we need a full call to ENSURE-SYMBOL-HASH
-           ;; to ensure the side-effect happens.
-           ;; Careful if changing this again. There'a regression test thank goodness.
-           (declare (notinline ensure-symbol-hash))
-           (ensure-symbol-hash symbol)
-           symbol))
+                             fasl-input))))
 
   (define-fop 77 :not-host (fop-lisp-symbol-save ((:operands length+flag)))
     (aux-fop-intern length+flag *cl-package* t (fasl-input)))
@@ -872,12 +859,11 @@
 
   (define-fop 80 :not-host (fop-uninterned-symbol-save ((:operands length+flag)))
     (multiple-value-bind (name len) (read-symbol-name length+flag (fasl-input))
-      (push-fop-table (ensure-hashed (make-symbol (subseq name 0 len)))
+      (push-fop-table (make-symbol (subseq name 0 len))
                       (fasl-input))))
 
   (define-fop 81 :not-host (fop-copy-symbol-save ((:operands table-index)))
-    (push-fop-table (ensure-hashed
-                     (copy-symbol (ref-fop-table (fasl-input) table-index)))
+    (push-fop-table (copy-symbol (ref-fop-table (fasl-input) table-index))
                     (fasl-input))))
 
 (define-fop 82 (fop-package (pkg-designator))
@@ -936,7 +922,7 @@
   #-sb-xc-host (%make-ratio num den))
 
 (define-fop 71 (fop-complex (realpart imagpart))
-  #+sb-xc-host (number-pair-to-core realpart imagpart sb-vm:complex-widetag)
+  #+sb-xc-host (number-pair-to-core realpart imagpart sb-vm:complex-rational-widetag)
   #-sb-xc-host (%make-complex realpart imagpart))
 
 (macrolet ((fast-read-single-float ()
@@ -1338,14 +1324,7 @@
         (#x6d sb-c::compiled-debug-info)
         (#x6e sb-c::debug-source)
         (#x6f defstruct-description)
-        (#x70 defstruct-slot-description)
-        (#x71 sb-c::debug-fun)
-        (#x72 sb-c::compiled-debug-fun)
-        (#x73 sb-c::compiled-debug-fun-optional)
-        (#x74 sb-c::compiled-debug-fun-more)
-        (#x75 sb-c::compiled-debug-fun-external)
-        (#x76 sb-c::compiled-debug-fun-toplevel)
-        (#x77 sb-c::compiled-debug-fun-cleanup)))
+        (#x70 defstruct-slot-description)))
 
 
 ;;;; stuff for debugging/tuning by collecting statistics on FOPs (?)
