@@ -113,18 +113,13 @@
 
 (defknown (%sxhash-simple-substring) (simple-string index index) hash-code
   (foldable flushable))
-(defknown sb-impl::calc-symbol-name-hash (simple-string index) hash-code ())
+(defknown sb-impl::calc-symbol-name-hash (simple-string index) symbol-name-hash ())
 
 (defknown (symbol-hash) (symbol) hash-code (flushable movable))
-;;; This unusual accessor will read the word at SYMBOL-HASH-SLOT in any
-;;; object, not only symbols. The result is meaningful only if the object
-;;; is a symbol. The second argument indicates a predicate that the first
-;;; argument is known to satisfy, if any.
-;;; There is one more case, but an uninteresting one: the object is known
-;;; to be anything except NIL. That predicate would be IDENTITY,
-;;; which is not terribly helpful from a code generation stance.
-(defknown symbol-hash* (t (member nil symbolp non-null-symbol-p))
-  hash-code (flushable movable always-translatable))
+(defknown (symbol-name-hash) (symbol) symbol-name-hash (flushable movable))
+;;; This accessor will read the half-lispword at SYMBOL-HASH-SLOT in any object,
+;;; not only symbols. The value is reliable only if the object is a symbol.
+(defknown hash-as-if-symbol-name (t) symbol-name-hash (flushable movable always-translatable))
 
 (defknown %set-symbol-hash (symbol hash-code)
   t ())
@@ -810,7 +805,7 @@
   (double-float (signed-byte 32)) double-float
   (movable foldable flushable))
 
-(defknown (%log1p)
+(defknown (%log1p %log2)
   (double-float) double-float
   (movable foldable flushable))
 
