@@ -167,9 +167,12 @@
 #+sb-simd-pack
 (progn
   (defconstant-eqx +simd-pack-element-types+
-      #(single-float double-float
-        (unsigned-byte 8) (unsigned-byte 16) (unsigned-byte 32) (unsigned-byte 64)
-        (signed-byte 8) (signed-byte 16) (signed-byte 32) (signed-byte 64))
+      (coerce
+       (hash-cons
+        '(single-float double-float
+          (unsigned-byte 8) (unsigned-byte 16) (unsigned-byte 32) (unsigned-byte 64)
+          (signed-byte 8) (signed-byte 16) (signed-byte 32) (signed-byte 64)))
+       'vector)
     #'equalp)
   (defconstant sb-kernel::+simd-pack-wild+
     (ldb (byte (length +simd-pack-element-types+) 0) -1))
