@@ -283,28 +283,6 @@
     (loadw result function closure-fun-slot fun-pointer-lowtag)
     (inst sub result (- (* simple-fun-insts-offset n-word-bytes) fun-pointer-lowtag))))
 
-;;;; symbol frobbing
-(defun load-symbol-dbinfo (result symbol)
-  (loadw result symbol symbol-info-slot other-pointer-lowtag)
-  ;; If RES has list-pointer-lowtag, take its CDR. If not, use it as-is.
-  ;; This CMOV safely reads from memory when it does not move, because if
-  ;; there is a PACKED-INFO in the slot, it has at least 4 data words in total
-  ;; - the header, at least one info descriptor, and at least one datum.
-  ;; And since 3 is odd, that would be aligned up to 4.
-  ;; Use bit index 2 of the lowtag to distinguish list from instance.
-  ;; An instance will have a 0 in that bit.
-  ;; This would compile to almost the same code without a VOP,
-  ;; but using a jmp around a mov instead.
-  (aver (= (logior instance-pointer-lowtag #b0100) list-pointer-lowtag))
-  (inst test :byte result #b0100)
-  (inst cmov :nz result (object-slot-ea result cons-cdr-slot list-pointer-lowtag)))
-
-(define-vop (symbol-dbinfo)
-  (:policy :fast-safe)
-  (:translate symbol-dbinfo)
-  (:args (x :scs (descriptor-reg)))
-  (:results (res :scs (descriptor-reg)))
-  (:generator 1 (load-symbol-dbinfo res x)))
 
 ;;;; other miscellaneous VOPs
 
@@ -533,3 +511,33 @@ number of CPU cycles elapsed as secondary value. EXPERIMENTAL."
     ;; but since this needs pseudo-atomic decoration anyway, it saves code size
     ;; to make an assembly routine that preserves all registers.
     (invoke-asm-routine 'call 'switch-to-arena vop)))
+
+;;; Caution: this vop potentially clobbers all registers, but it doesn't declare them.
+;;; It's safe to use only from DESTROY-ARENA which, being an ordinary full call,
+;;; is presumed not to preserve registers.
+#+sb-xc-host
+(define-vop (delete-arena)
+  (:args (x :scs (descriptor-reg)))
+  (:temporary (:sc unsigned-reg :offset rdi-offset :from (:argument 0)) rdi)
+  (:temporary (:sc unsigned-reg :offset rbx-offset) rsp-save)
+  (:vop-var vop)
+  (:generator 1
+    (move rdi x)
+    (pseudo-atomic ()
+      (inst mov rsp-save rsp-tn)
+      (inst and rsp-tn -16) ; align as required by some ABIs
+      (call-c "sbcl_delete_arena" #+win32 rdi)
+      (inst mov rsp-tn rsp-save))))
+
+#+ultrafutex
+(define-vop (quick-try-mutex)
+  (:translate quick-try-mutex)
+  (:policy :fast-safe)
+  (:args (m :scs (descriptor-reg)))
+  (:temporary (:sc unsigned-reg :offset 0) old) ; RAX
+  (:temporary (:sc unsigned-reg) new)
+  (:conditional :z)
+  (:generator 1
+    (inst mov :byte new 1)
+    (zeroize old)
+    (inst cmpxchg :lock :byte (mutex-slot m state) new)))
