@@ -489,12 +489,6 @@ static void print_slots(char **slots, int count, lispobj *ptr)
     }
 }
 
-lispobj symbol_function(struct symbol* symbol)
-{
-    if (symbol->fdefn) return FDEFN(symbol->fdefn)->fun;
-    return NIL;
-}
-
 static void print_fun_or_otherptr(lispobj obj)
 {
     int index;
