@@ -103,7 +103,7 @@ static int readonly_unboxed_obj_p(lispobj* obj)
     case SIMD_PACK_256_WIDETAG:
 #endif
         return 1;
-    case RATIO_WIDETAG: case COMPLEX_WIDETAG:
+    case RATIO_WIDETAG: case COMPLEX_RATIONAL_WIDETAG:
         return fixnump(obj[1]) && fixnump(obj[2]);
     }
     if ((widetag > SIMPLE_VECTOR_WIDETAG && widetag < COMPLEX_BASE_STRING_WIDETAG)) {
@@ -161,6 +161,7 @@ static void walk_all_gc_spaces(void (*fun)(lispobj*,uword_t), uword_t arg)
 {
     walk_range((lispobj*)NIL_SYMBOL_SLOTS_START, (lispobj*)NIL_SYMBOL_SLOTS_END, fun, arg);
     walk_range((lispobj*)STATIC_SPACE_OBJECTS_START, static_space_free_pointer, fun, arg);
+    walk_range((lispobj*)PERMGEN_SPACE_START, permgen_space_free_pointer, fun, arg);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     walk_range((lispobj*)FIXEDOBJ_SPACE_START, fixedobj_free_pointer, fun, arg);
 #endif
